/*
 * BSD License for xmlfy
 * Copyright (c) 2008-2010, Arthur Gouros
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * - Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 * - Neither the name of Arthur Gouros nor the names of its contributors 
 *   may be used to endorse or promote products derived from this software 
 *   without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 */

/**************************************************************************
 * xmlfy - Convert to XML on the fly.                                     *
 *         Original Author: Arthur Gouros.                                *
 *                                                                        *
 *         Please consult the documentation for further information.      *
 **************************************************************************
 */

#include "validate_args.h"

/* function templates */
int  process_arg_regex(unsigned char *opt_s, unsigned char *s);


int process_arg_regex(opt_s, s)
unsigned char *opt_s;
unsigned char *s;
{
  /* translate regex arg to column structure */
  unsigned char *opt_delimiter;
  int i, s_start, s_length, p_length, cflags;
  int ret;

  /* initialise */
  ret                   = TRUE;
  opt_delimiter         = NULL;
  s_start               = 0;
  s_length              = xstrlen(s);

  /* identify any quotes */
  if (s_length && (s[s_length - 1] == '\'' || s[s_length - 1] == '"'))
    s_length--;
  if (s_length && (s[0] == '\'' || s[0] == '"'))
  {
    s_start = 1;
    s_length--;
  }

  if (s_length == 0)
    ret = -21; /* invalid option "" - unknown. */

  /* determine the delimiter character used */
  i = s_start;
  while (i < s_length && s[i] != '/' && s[i] != '%')
    i++;
  if (i < s_length)
  {
    opt_delimiter = &s[i];
    s_start = i;
  }
  else
    ret = -22; /* invalid option - no option delimiter found. */

  /* check /<pattern>/ string components */
  for (p_length = 0 ; s_start + p_length + 1 < s_length && s[s_start + p_length + 1] != *opt_delimiter ; p_length++ )
    ;
  if (p_length == 0)
    ret = -23; /* invalid option - no pattern found. */
  else if (s[s_start + p_length + 1] != *opt_delimiter)
    ret = -22; /* invalid option - option delimiter mismatched. */

  /* check <cflags> string components */
  cflags = REG_EXTENDED; /* why bother with Basic REs */
  for (i = s_start + p_length + 2 ; i < s_length ; i++)
  {
    switch(s[i])
    {
      case 'i' : cflags |= REG_ICASE;
                 break;

      case 'l' : cflags |= REG_LITERAL;
                 break;

      case 'u' : cflags |= REG_UNGREEDY;
                 break;

      case 'r' : cflags |= REG_RIGHT_ASSOC;
                 break;

      default  : ret = -24; /* invalid CFLAGS */
    }
  }

  if (ret)
  {
    /* allocate memory for regex structure */
    if (ncolumn)
      column = (struct_column *) realloc(column, ((ncolumn + 1) * sizeof(struct_column)));
    else
      column = (struct_column *) malloc(sizeof(struct_column));
    if (column == NULL) out_of_mem();

    /* set the type of column */
    column[ncolumn].type = OPT_REGEX;

    column[ncolumn].cflags = cflags;
    column[ncolumn].pattern_length = p_length;
    column[ncolumn].pattern = (unsigned char *) malloc((column[ncolumn].pattern_length + 1) * sizeof(unsigned char));
    if (column[ncolumn].pattern == NULL) out_of_mem();
    for (i = 0 ; i < column[ncolumn].pattern_length ; i++)
      column[ncolumn].pattern[i] = s[s_start + 1 + i];
    column[ncolumn].pattern[i] = '\0';

    /* if scope exists process it */
    s_length = xstrlen(opt_s);
    for (s_start = 0 ; s_start < s_length && opt_s[s_start] != ':' ; s_start++ )
      ;
    if (s_start + 1 < s_length)
    {
      s_start++;
      s_length -= s_start;
      /* add scope to the regex_opt structure */
      if (populate_scope(OPT_REGEX, -1, &opt_s[s_start], s_length))
      {
        column[ncolumn].nscope = nscope;
        column[ncolumn].scope  = scope;
      }
      else
        ret = -15; /* invalid <scope> provided */
    }
    else
    {
      column[ncolumn].nscope = 0;
      column[ncolumn].scope  = NULL;
    }
    column[ncolumn].scope_seq = 0;
  }

  /* compile the RE pattern */
  if (ret)
  {
    column[ncolumn].pmatch = NULL;
    if (regncomp(&column[ncolumn].pcomp, (char *) column[ncolumn].pattern, column[ncolumn].pattern_length, column[ncolumn].cflags) == 0)
    {
      if (column[ncolumn].pcomp.re_nsub > 0)
        column[ncolumn].pmatch = (regmatch_t *) malloc((column[ncolumn].pcomp.re_nsub + 1) * sizeof(regmatch_t));
      else
        column[ncolumn].pmatch = (regmatch_t *) malloc(sizeof(regmatch_t));
      if (column[ncolumn].pmatch == NULL) out_of_mem();
    }
    else
      ret = -25; /* invalid RE pattern */

    ncolumn++;
  }

  return(ret);
}

